@@ -69,11 +69,35 @@ namespace Ogre {
             pFloat[3] = pPacked->w;
     }
 
-    static void copy_float3(uint8* pDst, uint8* pSrc, int elemOffset)
+    static void extract_float3(uint8* pDst, uint8* pSrc, int elemOffset)
     {
         memcpy(pDst, pSrc + elemOffset, sizeof(float) * 3);
     }
 
+    static void pad_16x3(uint8* pDst, uint8* pSrc, int elemOffset)
+    {
+        // for half3, we want 1.0 in the 4th component
+        // for others we dont care, so do it unconditionally
+        static const uint16 one16f = Bitwise::floatToHalf(1.0f);
+        memcpy(pDst + elemOffset, pSrc + elemOffset, sizeof(uint16) * 3);
+        memcpy(pDst + elemOffset + sizeof(uint16) * 3, &one16f, sizeof(uint16));
+    }
+
+    static void float_to_half_3(uint8* pDst, uint8* pSrc, int elemOffset)
+    {
+        float* pFloat = (float*)(pSrc + elemOffset);
+        uint16* pHalf = (uint16*)(pDst + elemOffset);
+        pHalf[0] = Bitwise::floatToHalf(pFloat[0]);
+        pHalf[1] = Bitwise::floatToHalf(pFloat[1]);
+        pHalf[2] = Bitwise::floatToHalf(pFloat[2]);
+    }
+
+    /** Splice out an element from a vertex buffer
+     * @param elem The element to splice out of the vertex
+     * @param srcBuf Source buffer
+     * @param pDst Destination buffer for the vertex without the element
+     * @param pElemDst Destination buffer for the element (can be the same as pDst)
+     */
     static void spliceElement(const VertexElement* elem, const HardwareVertexBufferPtr& srcBuf, uint8* pDst,
                               uint8* pElemDst, uint32 newElemSize, void (*elemConvert)(uint8*, uint8*, int))
     {
@@ -131,7 +155,7 @@ namespace Ogre {
             if (&e == elem)
             {
                 // Modify element
-                decl->modifyElement(idx, newSource, newElemOffset, newType, elem->getSemantic());
+                decl->modifyElement(idx, newSource, newElemOffset, newType, elem->getSemantic(), elem->getIndex());
             }
             else if (e.getSource() == oldSource && e.getOffset() > oldElemOffset)
             {
@@ -241,13 +265,17 @@ namespace Ogre {
         return dest;
     }
 
-    void VertexData::convertVertexElement(VertexElementSemantic semantic, VertexElementType dstType)
+    void VertexData::convertVertexElement(VertexElementSemantic semantic, VertexElementType dstType, uint16 index)
     {
-        auto elem = vertexDeclaration->findElementBySemantic(semantic);
+        auto elem = vertexDeclaration->findElementBySemantic(semantic, index);
 
-        if (!elem || VertexElement::getBaseType(elem->getType()) == VertexElement::getBaseType(dstType))
+        if(!elem)
             return; // nothing to do
 
+        auto srcBaseType = VertexElement::getBaseType(elem->getType());
+        if (srcBaseType == VET_FLOAT1 && srcBaseType == VertexElement::getBaseType(dstType))
+            return; // silently ignore floatX > floatY conversions
+
         auto srcType = elem->getType();
         auto vbuf = vertexBufferBinding->getBuffer(elem->getSource());
 
@@ -280,6 +308,17 @@ namespace Ogre {
                 OgreAssert(srcType == VET_INT_10_10_10_2_NORM, "unsupported conversion");
                 spliceElement(elem, vbuf, pDst, pDst, newElemSize, unpack_10_10_10_2<true>);
             }
+            else if(dstType == VET_HALF3)
+            {
+                OgreAssert(srcType == VET_FLOAT3, "unsupported conversion");
+                spliceElement(elem, vbuf, pDst, pDst, newElemSize, float_to_half_3);
+            }
+            else if(dstType == VET_HALF4 || dstType == VET_SHORT4 || dstType == VET_USHORT4)
+            {
+                // pad 16x3 formats to 16x4
+                OgreAssert(srcType == VET_HALF3 || srcType == VET_SHORT3 || srcType == VET_USHORT3, "unsupported conversion");
+                spliceElement(elem, vbuf, pDst, pDst, newElemSize, pad_16x3);
+            }
             else
             {
                 OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "unsupported dstType");
@@ -340,7 +379,7 @@ namespace Ogre {
         {
             // Basically we just memcpy the vertex excluding the position
             HardwareBufferLockGuard destRemLock(newRemainderBuffer, HardwareBuffer::HBL_DISCARD);
-            spliceElement(posElem, vbuf, (uint8*)destRemLock.pData, (uint8*)pDest, posElem->getSize(), copy_float3);
+            spliceElement(posElem, vbuf, (uint8*)destRemLock.pData, (uint8*)pDest, posElem->getSize(), extract_float3);
         }
         else
         {
@@ -511,12 +550,9 @@ namespace Ogre {
             for (VertexElement& destelem : destElems)
             {
                 // get source
-                const VertexElement* srcelem =
-                    vertexDeclaration->findElementBySemantic(
-                        destelem.getSemantic(), destelem.getIndex());
-                // get buffer
-                HardwareVertexBufferSharedPtr srcbuf = 
-                    vertexBufferBinding->getBuffer(srcelem->getSource());
+                auto srcelem = vertexDeclaration->findElementBySemantic(destelem.getSemantic(), destelem.getIndex());
+                OgreAssert(srcelem, "Semantic not found in existing declaration");
+                const auto& srcbuf = vertexBufferBinding->getBuffer(srcelem->getSource());
                 // improve flexibility only
                 if (srcbuf->getUsage() & HBU_CPU_ONLY)
                 {
@@ -989,6 +1025,8 @@ namespace Ogre {
         }
 
         indexBuffer->unlock();
+
+        triangles += indexBuffer->getNumIndexes()/3;
     }
 
     //-----------------------------------------------------------------------
